import { BadRequestException, Injectable, InternalServerErrorException, NotFoundException } from "@nestjs/common";
import * as crypto from "node:crypto";
import { readFile, writeFile } from 'node:fs/promises';
import { join } from "node:path";
import * as console from "node:console";
import FileType from 'file-type';

@Injectable()
export class UploadsService {

  //TODO save avatar under avatar-{userId}-{checksum}
  private async saveAvatar(userId: string, file: Buffer) {
    const checksum = await this.checkConditions(file)
    const fileType = await FileType.fileTypeFromBuffer(file)
    const fileName = `avatar-${userId}.${fileType.ext}`;
    await this.saveFile(fileName, file)
  }
  //TODO get
  private async getAvatar(userId: string) {

  }

  //TODO save product image under product-{productId}-{checksum}
  async saveProductImage(productId: string, imageBuffer: Buffer): Promise<string> {

  }
  //TODO get
  async getProductImage(imageId: string): Promise<Buffer> {

  }

  private getChecksum(file: Buffer) {
    return crypto.createHash('sha256').update(file).digest('hex').toLowerCase();
  }

  private async saveFile(fileName: string, file: Buffer) {
    try {
      await writeFile(join(process.cwd(), 'files/', fileName), file, 'utf8');
    } catch (err) {
      console.error(err);
      throw new InternalServerErrorException("File save failed !")
    }
  }

  private async checkConditions(file: Buffer) {
    const checksum = this.getChecksum(file)
    const fileType = await FileType.fileTypeFromBuffer(file)
    if (!fileType || !fileType.mime.startsWith('image/')) {
      throw new BadRequestException('Invalid file type. Only images are allowed.');
    }
    //check file size is less than 2mb
    const fileSize = file.byteLength;
    if (fileSize > 2 * 1024 * 1024) {
      throw new BadRequestException('File size exceeds the limit. Maximum file size allowed is 2MB.');
    }
    return checksum;
  }

  private async getFile(fileName: string){
    try {
      return await readFile(join(process.cwd(), 'files/', fileName));
    } catch (err) {
      throw new NotFoundException("File not found")
    }
  }

}
